class LearningObject
  include ActiveModel::Model
  attr_accessor :id, :id_dspace, :rid, :name, :description,
                :thumbnail, :date_creation, :last_modified,
                :type, :bitstreams, :metadata, :likes, :views, :downloads

  validates_presence_of :name, :date_creation, :type, :likes, :views, :downloads
  validates_numericality_of :likes, greater_than_or_equal_to: 0
  validates_numericality_of :views, greater_than_or_equal_to: 0
  validates_numericality_of :downloads, greater_than_or_equal_to: 0

  def initialize(params={})
    super(params.merge(defaults))
  end

  def subjects
    @subjects ||= learning_object_repository.get_subjects(self)
  end

  def get_metadata_value_of key
    values = @metadata.select { |v| v["key"] == key }
    unless values.empty?
      return values.first["value"]
    end
  end

  private

  def defaults
    {
        likes: 0,
        views: 0,
        downloads: 0,
        date_creation: DateTime.new
    }
  end

  def learning_object_repository
    Portalmec::Application.repository.for :learning_object
  end

end