module OrientDb
  class LearningObjectRepository < Base
    include OrientDb::Methods::EdgeMethods

    def increment_views(user, learning_object)
      create_edge "Views", user.rid, learning_object.id
    end

    #
    # Usage:
    #   repository.for(:learning_objects).like @learning_object, current_user
    #
    def like(user, learning_object)
      create_edge "Likes", user.rid, learning_object.id
    end

    # Example:
    #   list = repository.for(:learning_objects).all
    #   list.each do |learning_object|
    #     learning_object.inspect <LearningObject model>
    #   end
    def all
      learning_objects_hash = connection.query "SELECT FROM LearningObject"
      learning_objects = build_learning_objects(learning_objects_hash) || []
    end

    # Usage:
    #   learning_object = repository.for(:learning_objects).get_by_dspace_id 123
    #
    def get_by_dspace_id(id_dspace)
      result = connection.query "SELECT FROM LearningObject WHERE id_dspace=#{id_dspace}"
      build_learning_object result.first
    end

    # Usage:
    #   learning_object = repository.for(:learning_objects).get_by_dspace_id 123
    #
    def find(id)
      result = connection.query "SELECT FROM #{id}"
      build_learning_object result.first
    end

    def create(name, url)
      connection.command sprintf("INSERT INTO LearningObject (name,URL) VALUES ('%s','%s')", name, url)
    end

    # Usage:
    #   repository.for(:learning_objects).destroy learning_object
    #
    def destroy(learning_object)
      connection.command sprintf("DELETE VERTEX LearningObject where @rid = '%s'", learning_object.id)
    end

    #def author_of(rid)
    #  connection.query "SELECT expand(in) FROM (SELECT expand(out_author_of) FROM User WHERE @rid=#{rid})"
    #end

    #def has(rid)
    #  connection.query "SELECT expand(in) FROM (SELECT expand(out_has) FROM User WHERE @rid=#{rid})"
    #end

    #def get_likes(id)
    #  likes = connection.query "select outE('Likes') from #{id}"
    #  likes[0]["outE"]
    #end

    private

    def count_likes(learning_object)
      get_in_edges_count "Likes", learning_object.id
    end

    def count_views(learning_object)
      get_in_edges_count "Views", learning_object.id
    end

    def build_learning_object(args={})
      lo = LearningObject.new(:id => args["@rid"],
                              :name => args["name"],
                              :description => args["description"],
                              :thumbnail => args["thumbnail"],
                              :date_creation => args["date_creation"],
                              :id_dspace => args["id_dspace"],
                              :type => args["type"],
                              :bitstreams => args["bitstreams"],
                              :metadata => args["metadata"],
                              :last_modified => args["last_modified"])

      lo.likes = count_likes lo
      lo.views = count_views lo
      lo
    end

    def build_learning_objects(hash=[])
      learning_objects = []
      hash.each do |h|
        learning_objects << build_learning_object(h)
      end
      learning_objects
    end

  end
end
