class DspaceService
  attr_reader :dspace_client
  @@clients = {}

  def initialize(dspace_client)
    @dspace_client = dspace_client
  end

  def self.create_dspace_client(url = Dspace::Config.rest_url, auth_token=nil)
    @@clients[url] ||= DspaceClient.new(url, auth_token)
  end

  def get_object_by_id(id)
    item_repository = @dspace_client.repository.item_repository

    item = item_repository.get_item_by_id(id, "all")
    metadata = metadata2hash(item)
    build_item(item, metadata)
  end

  def get_bitstream_by_id(id)
    bitstream_repository = @dspace_client.repository.bitstream_repository

    bitstream = bitstream_repository.get_bitstream_by_id(id, "parent")
    build_bitstream(bitstream)
  end

  def retrieve_bitstream(bitstream)
    bitstream_repository = @dspace_client.repository.bitstream_repository

    bitstream_repository.retrieve_data(bitstream)
  end

  def create_bitstream(item, bitstream, file, auth_token = nil)
    if auth_token.nil?
      raise 'You must be authenticated to create a bitstream.'
    end

    item_repository = @dspace_client.repository.item_repository
    bitstream_repository = @dspace_client.repository.bitstream_repository
    upload_strategy = DSpaceRest::Strategies::Uploads::CurlStrategy.new(@dspace_client.url, auth_token)

    # upload file
    response = item_repository.create_bitstream_for(item, file, upload_strategy)

    # update bitstream information
    response.name = bitstream["name"]
    response.description = bitstream["description"]
    bitstream_repository.update(response)
  end

  def create_object(disciplina_id, form_item)
    collection_repository = @dspace_client.repository.collection_repository
    disciplina = collection_repository.get_collection_by_id(disciplina_id)

    # trata o form da item
    item = DSpaceRest::Item.new(form_item)
    dspace_keys = get_dspace_metadata_names("invert")
    form_item.each do |k,v|
      unless dspace_keys[k].nil?
        item.add_metadata(dspace_keys[k], v, "pt-BR")
      end
    end

    #manda pro dspace
    collection_repository.create_item_for(disciplina, item)
  end

  def get_dspace_metadata_names(invert=nil)
    h = {
        "dc.title" => "name",
        "dc.contributor.author" => "author_id",
        "dc.description" => "description",
        "dc.date.issued" => "date",
        "dc.rights" => "license_name",
        "dc.rights.uri" => "license_uri"
    }
    if (invert != nil)
      return h.invert
    end
    h
  end

  private

  def build_object(item, metadata={})
    item.new({
                 id: item.id,
                 name: item.name,
                 # TODO: real professor
                 author_id: metadata["author_id"],
                 description: metadata["description"] || "",
                 date: metadata["date"] || "",
                 license_name: metadata["license_name"] || "",
                 license_uri: metadata["license_uri"] || "",
                 disciplina: item.parent_collection,
                 departamento: item.parent_community_list[0],
                 setor: item.parent_community_list[1],
                 files: item.bit_streams
             })
  end

  def build_bitstream(bitstream)
    Bitstream.new({
                      id: bitstream.id,
                      name: bitstream.name,
                      description: bitstream.description,
                      retrieve_link: Rails.application.secrets.dspace_rest_url + bitstream.retrieve_link,
                      size_bytes: bitstream.size_bytes
                  })
  end

  def metadata2hash(item)
    h = {}
    hkeys = get_dspace_metadata_names

    item.metadata.each do |m|
      unless hkeys[m.key].nil?
        h[hkeys[m.key]] = m.value
      end
    end

    h
  end
end
